//////////////////////////////////////////////////
// (c) MMV Sven Reichard
// 

# include <OpeningBookNode.h>
# include <Moves/SanNotation.h>
# include <Game/Game.h>
# include <Algorithms/HistoryAlphaBeta.h>
# include <RealTimer.h>
# include <Game/PositionalEvaluation.h>
# include <iostream>
# include <fstream>
OpeningBookNode::OpeningBookNode()
  : treeValue( Evaluation::mate() ), alternativeValue(Evaluation::mate()),
    opponentValue( -Evaluation::mate() ),
    move("-"),
    isWon( false ),
    parent( 0 ), next( 0 ), first( 0 ),
    hasAlternative( false )
{

};

OpeningBookNode::~OpeningBookNode()
{
  delete next;
  delete first;
};

void
OpeningBookNode::addFirst( OpeningBookNode* node)
{
  node->next = first;
  first = node;
  node->parent = this;
};

void
OpeningBookNode::update()
{
  treeValue = alternativeValue;
  if (!first) return;
  isWon = false;
  opponentValue = Evaluation::mate();
  treeValue = Evaluation::mate();
  if (hasAlternative)
    treeValue = alternativeValue;
  for (OpeningBookNode* p = first;p; p = p->next)
    {
      if (!p->isWon)
	isWon = true;
      Evaluation::Score newValue = -(p->treeValue);
      if (newValue > opponentValue)
	opponentValue = newValue;
      newValue = -(p->opponentValue);
      if (newValue > treeValue)
	treeValue = newValue;
    }
};

void
OpeningBookNode::updateAll()
{
  update();
  if (parent)
    parent->updateAll();
};

void 
OpeningBookNode::setAlternative( const std::string& alt,
				 Evaluation::Score value )
{
  alternative = alt;
  alternativeValue = value;
  hasAlternative = true;
};

void
OpeningBookNode::setPosition(Game& g) const
{
  if (move == "-")
    return;
  if (parent)
    parent->setPosition(g);
  SanNotation san(move, g);
  san.getMove()->makeOn(g);
};

void
OpeningBookNode::findAllAlternatives( double seconds )
{
  if (!hasAlternative)
    findAlternative(seconds);
  
  if (first)
    first->findAllAlternatives(seconds);
  if (next)
    next->findAllAlternatives(seconds);
};

bool
OpeningBookNode::findAlternative( double seconds )
{
  if (hasAlternative)
    return false;
  Game g;
  setPosition(g);
  MoveList allMoves;
  g.pseudoLegalMoves(allMoves);
  MoveList moves;
  for (MoveList::iterator move = allMoves.begin();
       move != allMoves.end(); ++ move )
    {
      bool isNew = true;
      (*move)->makeOn(g);
      bool inCheck = g.isInCheck(g.colorToMove()->otherColor());
      (*move)->takeBackOn(g);
      if (inCheck)
	continue;
      SanNotation san(*move, g);
      for (OpeningBookNode* node = first; node; node = node->next)
	if (node->move == san.getString())
	  {
	    isNew = false;
	    break;
	  }
      if ( isNew )
	moves.push_back(*move);
    }
  if (moves.empty())
    return false;
  std::ofstream logFile("opening.log", std::ios_base::app);
  logFile<<"investigating line: "<<std::endl;
  printLine(logFile);
  logFile<<std::endl;
  if (isWon)
    logFile<<"won position"<<std::endl;
  else
    logFile<<"current value: "<<treeValue<<std::endl;
  logFile<<"children:"<<std::endl;
  for (OpeningBookNode* child = first; child; child = child->next)
    {
      logFile<<" "<<child->move<<"  ";
      if (! child->isWon)
	logFile<<"won!";
      else
	logFile<<-child->opponentValue;
      logFile<<std::endl;
    }
  HistoryAlphaBeta algo(g);
  algo.setEvaluation(new PositionalEvaluation);
  RealTimer timer;
  int greatestDepth;
  for (int depth = 1; timer.age() < seconds; depth++)
    {
      SmartPointer<Move> bestMove = algo.bestMove();
      if (bestMove)
	{
	  MoveList::iterator it = moves.begin();
	  while (it != moves.end() && **it != *bestMove)
	    ++ it;
	  if (it != moves.end())
	    {
	      moves.erase(it);
	      moves.insert(moves.begin(),bestMove);
	    }
	}
      algo.searchFixedDepth(depth, moves);
      greatestDepth = depth;
    }
  
  SanNotation san(algo.bestMove(), g);
  logFile<<"depth "<<greatestDepth<<std::endl;
  logFile<<"found move "<<san.getString()<<", value "<<algo.score()<<std::endl;
  logFile<<"-----"<<std::endl<<std::endl;
  setAlternative( san.getString(), algo.score());
  updateAll();
  return true;
};

void
OpeningBookNode::printOn( std::ostream& aStream ) const
{
  aStream << std::endl<<"( ";
  aStream << move << " ";
  aStream << isWon << " ";
  aStream << treeValue<< " ";
  aStream << opponentValue << " ";
  aStream << hasAlternative << " ";
  if (hasAlternative)
    {
      aStream << alternativeValue << " ";
      aStream << alternative << " ";
    }
  if (first)
    first->printOn( aStream );
  aStream << ") ";
  if (next)
    next->printOn(aStream);
};

void
OpeningBookNode::readFrom( std::istream& aStream )
{
  char dummy;
  aStream >> dummy; // '('
  aStream >> move;
  aStream >> isWon;
  aStream >> treeValue;
  aStream >> opponentValue;
  aStream >> hasAlternative;
  if (hasAlternative)
    aStream >> alternativeValue >> alternative;
  while( aStream >> dummy)
    {
      if (dummy != '(')
	{
	  if (dummy != ')')
	    std::cout<<"error: "<<dummy<<std::endl;
	  return;
	}
	
      aStream.putback(dummy);
      OpeningBookNode* child = new OpeningBookNode;
      child->readFrom(aStream);
      addFirst(child);
      
    }
  
};

bool
OpeningBookNode::findOneAlternative( int level, double seconds )
{
  if (level > 0)
    {
      if ((first!= 0) && first->findOneAlternative( level-1, seconds ))
	return true;
    }
  else
    if (findAlternative(seconds))
      return true;
  if (next && next->findOneAlternative( level, seconds ))
    return true;
  return false;
};

void
OpeningBookNode::printLine( std::ostream& out )
{
  if (!parent)
    return;
  parent->printLine(out);
  out<<move<<" ";
};

int
OpeningBookNode::numberOfLines() const
{
  int result = 1;
  if (first)
    result = first->numberOfLines();
  if (next)
    result += next->numberOfLines();
  return result;
};

int
OpeningBookNode::numberOfMoves() const
{
  int result = 1;
  if (first)
    result += first->numberOfMoves();
  if (next)
    result += next->numberOfMoves();
  return result;
};
