//////////////////////////////////////////////////
// (c) MMIII Sven Reichard
// 

# include <XBoard/XBoardInterface.h>
# include <XBoard/XBoardFeature.h>
# include <iostream>
# include <Game/Game.h>
# include <Game/GameRecord.h>
# include <Algorithms/HistoryAlphaBeta.h>
# include <Game/PositionalEvaluation.h>
# include <Moves/SanNotation.h>
# include <sstream>
# include <TestData/TestPositionSuite.h>
# include <signal.h>
# include <iterator>
# include <OpeningBook.h>
# include <unistd.h>

#include <ctime>

using namespace Alice;
using namespace Alice::XBoard;
bool
XBoardInterface::idle;
OpeningBook* 
XBoardInterface::book;
XBoardInterface::XBoardInterface()
  : XBoardBase( std::cin, std::cout )
{
  initialize();
};
XBoardInterface::XBoardInterface(std::istream& in, std::ostream& out)
  : XBoardBase( in, out ),
    logFile("Alice.log")
{
  initialize();
};
void
XBoardInterface::initialize()
{
  game = 0;
  algorithm = 0;
  evaluation = 0;
  //  record = new GameRecord;
  XBoardBase::initialize();
  idle = true;
  #ifndef SA_RESTART
  #define SA_RESTART 0
  #endif
  #ifdef SIGALRM
  int flags = SA_RESTART;
  struct sigaction action;
  action.sa_handler = signalHandler;
  action.sa_flags = flags;
  sigaction(SIGALRM, &action, 0);
  alarm(60);
  #endif
  book = new OpeningBook;
  book->open("opening.opn");
  
};

XBoardInterface::~XBoardInterface()
{
  delete game;
  delete algorithm;
  delete evaluation;
  //delete record;
  for (std::map<std::string, XBoardFeature*>::iterator it = features.begin();
       it != features.end(); ++ it)
    delete it->second;
};


Game*
XBoardInterface::getGame()
{
  return game;
};

SmartPointer<Move>
XBoardInterface::readMove( const std::string& buffer )
{
  if (*getFeature("san"))
    {
      SanNotation notation(buffer, *game);
      return notation.getMove();
    }
  MoveList list;
  game->pseudoLegalMoves(list);
  MoveList::const_iterator it;
  for ( it = list.begin();
	it != list.end(); ++ it)
    if ((*it)->fitsDescription(buffer))
      {
	return *it;
      }
  return 0;
};

std::string
XBoardInterface::moveToString(SmartPointer<Move> move)
{
  if (*getFeature("san"))
    {
      SanNotation san(move, *game);
      return san.getString();
    };
  std::ostringstream out;
  move->printOn(out);
  return out.str();
};

void
XBoardInterface::noOperation( )
{

};

void
XBoardInterface::go()
{
  mode = GO;
  search();
};

void
XBoardInterface::newGame()
{
  delete game;
  game = new Game;
  delete algorithm;
  algorithm = new HistoryAlphaBeta(*game);
  delete evaluation;
  evaluation = new PositionalEvaluation;
  algorithm -> setEvaluation( evaluation );
  record = new GameRecord;
  oppComputer = false;
  mode = GO;
  movesSoFar = 0;
  learningFile.open("position.epd");
  lastDepth = 0;
  actualPositions.clear();
  if (book)
    book->reset();
  
};

void
XBoardInterface::usermove()
{
  std::string moveString;
  input >> moveString;
  SmartPointer<Move> move = readMove(moveString);
  if (!move)
    {
      game->board().displayOn(std::cerr);
      if (game->colorToMove()->isWhite())
	std::cerr<<"White to move"<<std::endl;
      else
	std::cerr<<"Black to move"<<std::endl;
    }
  if (record)
    record->addMove(move);
  bool wasPredicted = predictedMove && (*move == *predictedMove);
  if (wasPredicted)
    {
      logFile << "predicted move was made"<<std::endl;
    }
  else if (predictedMove)
    {
      logFile << "predicted move was: ";
      predictedMove->printOn(logFile);
      logFile << std::endl;
    }
  if (!leafPositions.empty())
    leafPositions.back().setOperand("predicted",
				    wasPredicted? "true":"false");
  makeMove(move);
  
  if (getMode() == GO)
    search();
};

void
XBoardInterface::processCommand()
{
  std::string command;
  
  input >> command;
  if (! input )
    return;
  //  std::cerr<<"processing command \""<<command<<"\""<<std::endl;
  Command c = commands[command];
  if (c)
    (this->*c)();

  else if (game && !*getFeature("usermove"))
    {
      SmartPointer<Move> move = readMove(command);
      if (! move)
	{
	  output << "Error (unknown command): " << command <<std::endl;
	    return;
	}
      if (record)
	record->addMove(move);
      makeMove(move);
      if (getMode() == GO)
	search();
    }
  else
    output << "Error (unknown command): " << command <<std::endl;
};

double
XBoardInterface::timeToSearch() const
{
  return timeControl.timeToSearch( movesSoFar );
};

void
XBoardInterface::reportResult()
{

  assert(game->isStalemate());
  logFile << "claiming stalemate" << std::endl;
  if (game->isInCheck(game->colorToMove()))
    {
      if (game->colorToMove() == Color::white())
	output<<"result 0-1 {Black mates}\n";
      else
	output<<"result 1-0 {White mates}\n";
      return;
    }
  output<<"result 1/2-1/2 {Stalemate}\n";
  return;
    
};

void
XBoardInterface::checkTactics()
{
  
  if (actualPositions.size() > 1)
    {
      if ((lastScore > -2000) &&(algorithm->score() < lastScore - 1000))
	{
	  std::ofstream tactics("tactics.epd", std::ios::app);
	  actualPositions[actualPositions.size()-2].writeEPD(tactics);
	  actualPositions[actualPositions.size()-1].writeEPD(tactics);

	}
    }
  lastScore = algorithm->score();
  
};

void
XBoardInterface::writeLeafPosition( std::vector<SmartPointer<Move> >&
				    pv)
{

  for (std::vector<SmartPointer<Move> >::iterator move = pv.begin();
       move != pv.end(); ++ move)
    (*move)->makeOn(*game);
  TestPosition leafPosition(*game);
  leafPosition.writeEPD(learningFile);
  leafPositions.push_back(leafPosition);
  for (std::vector<SmartPointer<Move> >::reverse_iterator move = 
	 pv.rbegin();
       move != pv.rend(); ++ move)
    (*move)->takeBackOn(*game);
};

void
XBoardInterface::sendMove(SmartPointer<Move> move)
{
  static int badPositions = 0;
  
  if (move && algorithm->score() < - 9* evaluation->getBasicValue())
    {
      badPositions ++;
      logFile<<"bad positions: "<<badPositions<<std::endl;
      if (badPositions > 3)
	{
	  output<<"resign"<<std::endl;
	  return;
	}
    }
  else
    badPositions = 0;
  output<<"move "<<moveToString(move)<<std::endl;
  makeMove(move);
  if (record)
    record->addMove(move);
    
};

void
XBoardInterface::search()
{
  idle = false;
  if (! game)
    return;
  if (game->isStalemate())
    {
      reportResult();
      return;
    }
  movesSoFar ++;
  if (movesSoFar >= getMovesPerSession())
    movesSoFar -= getMovesPerSession();
  if (!book)
    logFile<<"no book!"<<std::endl;
  else
    {
      std::string suggestion = book->suggestMove();
      SanNotation san(suggestion, *game);
      SmartPointer<Move> move = san.getMove();
      if (move)
	{
	  logFile<<"suggestion: "<<suggestion<<std::endl;
	  sendMove(move);
	  return;
	}
    }
  
  myColor = game->colorToMove();
  algorithm->setGame(*game);
  algorithm->clearTable();
  double time = timeToSearch();
  algorithm->searchTime(time, logFile);

  std::ostringstream pvStream;
  algorithm->printCompletePV(algorithm->getSearchDepth(), pvStream);

  logFile<<"PV: " << pvStream.str() << std::endl;
  TestPosition currentPosition(*game);
  currentPosition.setOperand("pv", pvStream.str());
  actualPositions.push_back(currentPosition);
  checkTactics();
  std::vector<SmartPointer<Move> > pv =
    algorithm->completePV(algorithm->getSearchDepth());
  if (pv.size() > 1)
    predictedMove = pv[1];
  writeLeafPosition( pv );
  sendMove(algorithm->bestMove());
};

void
XBoardInterface::saveRecord()
{
  if (! record)
    return;
  record->setTag("Result", getResult());
  std::string ownColor("White");
  std::string otherColor("Black");
  
  if (myColor->isBlack())
    std::swap(ownColor, otherColor);
 
  record->setTag(ownColor, "Alice");
  record->setTag(otherColor, getOpponentName());
  if (getMyRating())
    record->setTag(ownColor + "Elo", getMyRating() );
  if (getOpponentRating())
    record->setTag(otherColor + "Elo", getOpponentRating() );
  time_t t = std::time(0);
  struct tm* l = localtime(&t);
  std::ostringstream date;
  date << l->tm_year+1900 << "." 
       << l->tm_mon+1 << "." 
       << l->tm_mday;
  record->setTag("Date", date.str());
  record->setTag("TimeControl", timeControl.getPgnString());

  std::ofstream output("alice.pgn", std::ios_base::app);
  record->write(output);
  output.close();

};

void
XBoardInterface::learn()
{
  learningFile.close();
  PositionalEvaluation eval;
  std::ofstream learnLog("learn.log", std::ios_base::app);
  TestPositionSuite suite("leafPositions.epd");
  learnLog << "found " << suite.numberOfPositions()
	   << " positions" << std::endl;
  if (suite.numberOfPositions() == 0)
    return;
  std::vector<Evaluation::Score> scores(suite.numberOfPositions());
  eval.computeScores( suite, scores );
  Evaluation::Score error = eval.totalError( scores );
  learnLog<<"total error: "<<error<<std::endl;
  eval.adjustWeights(suite,scores, myColor);
  eval.computeScores(suite, scores);
  learnLog<<"total error now: "<<eval.totalError(scores)<<std::endl;
};

void
XBoardInterface::result()
{
  XBoardBase::result();
  std::ofstream leaves("leafPositions.epd", std::ios_base::trunc);
  for (std::vector<TestPosition>::const_iterator it =
	 leafPositions.begin(); it !=  leafPositions.end(); ++ it)
    it -> writeEPD( leaves );
  leaves.close();
  leafPositions.clear();
  saveRecord();
  learn();

  std::cout<<"tellics set 7 Opening book: currently "
	   <<book->numberOfMoves()<<" moves in "
	   <<book->numberOfLines()<<" lines."<<std::endl;  
  book->save("opening.opn");
  idle = true;
};

void
XBoardInterface::setboard()
{
  learningFile.open("position.epd");
  TestPosition pos;
  pos.readEPD(input);
  delete game;
  game = new Game;
  //  std::cout<<"forsythe: "<<pos.forsythe()<<std::endl;
  game->forsytheString(pos.forsythe());
  if (pos.getColorToMove() == "b")
    game->doNullMove();
  //game->board().displayOn(std::cout);
  delete record;
  record = 0;
};

void
XBoardInterface::undo()
{

  moves.top()->takeBackOn(*getGame());
  moves.pop();
  if (record)
    record->backward();

  
};

void
XBoardInterface::makeMove(SmartPointer<Move> move)
{
  SanNotation san(move, *getGame());
  if (book)
    book->makeMove(san.getString());
  moves.push(move);
  move->makeOn(*getGame());
};

void
XBoardInterface::remove()
{
  undo();
  undo();
};

void
XBoardInterface::board()
{
  getGame()->board().displayOn(output);
};


void
XBoardInterface::signalHandler(int )
{

  if (!idle)
    {
     #ifdef SIGALRM
	 alarm(60);
     #endif
      return;
    }
  int level = book->findOneAlternative(5);
  std::cout<<"tellics resume"<<std::endl;
  if (level >= 0)
    {
      book->save("opening.opn");
      #ifdef SIGALRM
	  alarm(30);
      #endif
      return;
    }
  #ifdef SIGALRM
  alarm(120);
  #endif
};

bool
XBoardInterface::shouldQuitSearch()
{
  return false;
};
